@@ -1,27 +1,18 @@
-"""Sunsynk hybrid inverter sensor definitions."""
-from typing import Dict, Final, List
-
-from sunsynk.sensor import (
+"""Sunsynk 5kW&8kW hybrid inverter sensor definitions."""
+from sunsynk import AMPS, CELSIUS, KWH, VOLT, WATT
+from sunsynk.rwsensors import NumberRWSensor, SelectRWSensor, TimeRWSensor
+from sunsynk.sensors import (
     FaultSensor,
     InverterStateSensor,
     MathSensor,
-    NumberRWSensor,
     SDStatusSensor,
-    SelectRWSensor,
     Sensor,
     SerialSensor,
     TempSensor,
-    TimeRWSensor,
 )
 
-CELSIUS: Final = "°C"
-KWH: Final = "kWh"
-AMPS: Final = "A"
-VOLT: Final = "V"
-WATT: Final = "W"
-
-_SENSORS: List[Sensor] = []
-DEPRECATED: Dict[str, Sensor] = {}
+_SENSORS: list[Sensor] = []
+DEPRECATED: dict[str, Sensor] = {}
 
 ##########
 # Battery
@@ -89,7 +80,12 @@ _SENSORS += (
 ###################
 _SENSORS += (
     Sensor(166, "AUX power", WATT, -1),
-    MathSensor((175, 167, 166), "Essential power", WATT, factors=(1, 1, -1)),
+    # https://github.com/kellerza/sunsynk/issues/75
+    #  https://powerforum.co.za/topic/8646-my-sunsynk-8kw-data-collection-setup/?do=findComment&comment=147591
+    MathSensor(
+        (175, 169, 166), "Essential power", WATT, factors=(1, 1, -1), absolute=True
+    ),
+    # MathSensor((175, 167, 166), "Essential power", WATT, factors=(1, 1, -1)),
     MathSensor(
         (172, 167), "Non-Essential power", WATT, factors=(1, -1), no_negative=True
     ),
@@ -128,13 +124,15 @@ _SENSORS += (
 # General
 ##########
 RATED_POWER = Sensor((16, 17), "Rated power", WATT, 0.1)
+SERIAL = SerialSensor((3, 4, 5, 6, 7), "Serial")
 _SENSORS.append(RATED_POWER)
 _SENSORS += (
+    RATED_POWER,
+    SERIAL,
     Sensor(0, "Device Type"),
-    FaultSensor((103, 104, 105, 106, 107), "Fault"),
+    FaultSensor((103, 104, 105, 106), "Fault"),
     InverterStateSensor(59, "Overall state"),
     SDStatusSensor(92, "SD Status", ""),  # type: ignore
-    SerialSensor((3, 4, 5, 6, 7), "Serial"),
     TempSensor(90, "DC transformer temperature", CELSIUS, 0.1),
     TempSensor(95, "Environment temperature", CELSIUS, 0.1),
     TempSensor(91, "Radiator temperature", CELSIUS, 0.1),
@@ -145,7 +143,6 @@ _SENSORS += (
 ###########
 _SENSORS += (
     Sensor(200, "Control Mode"),
-    Sensor(230, "Grid Charge Battery current", AMPS, -1),
     Sensor(232, "Grid Charge enabled", "", -1),
     Sensor(312, "Battery charging voltage", VOLT, 0.01),
     Sensor(603, "Bat1 SOC", "%"),
@@ -213,6 +210,13 @@ _SENSORS += (
 _SENSORS.append(
     SelectRWSensor(243, "Priority Mode", options={0: "Battery first", 1: "Load first"})
 )
+_SENSORS.append(
+    SelectRWSensor(
+        244,
+        "Load Limit",
+        options={0: "Allow Export", 1: "Essentials", 2: "Zero Export"},
+    )
+)
 PROG1_TIME = TimeRWSensor(250, "Prog1 Time")
 PROG2_TIME = TimeRWSensor(251, "Prog2 Time", min=PROG1_TIME)
 PROG3_TIME = TimeRWSensor(252, "Prog3 Time", min=PROG2_TIME)
@@ -228,11 +232,18 @@ PROG5_TIME.max = PROG6_TIME
 PROG6_TIME.max = PROG1_TIME
 
 PROG_CHARGE_OPTIONS = {
-    4: "No Grid or Gen",
-    5: "Allow Grid",
-    6: "Allow Gen",
-    7: "Allow Grid & Gen",
+    0: "No Grid or Gen",
+    1: "Allow Grid",
+    2: "Allow Gen",
+    3: "Allow Grid & Gen",
 }
+PROG_MODE_OPTIONS = {
+    0: "None",
+    4: "General",
+    8: "Backup",
+    16: "Charge",
+}
+
 PROGRAM = (
     PROG1_TIME,
     PROG2_TIME,
@@ -252,12 +263,18 @@ PROGRAM = (
     NumberRWSensor(271, "Prog4 Capacity", "%", min=BATTERY_LOW_CAPACITY),
     NumberRWSensor(272, "Prog5 Capacity", "%", min=BATTERY_LOW_CAPACITY),
     NumberRWSensor(273, "Prog6 Capacity", "%", min=BATTERY_LOW_CAPACITY),
-    SelectRWSensor(274, "Prog1 Charge", options=PROG_CHARGE_OPTIONS),
-    SelectRWSensor(275, "Prog2 Charge", options=PROG_CHARGE_OPTIONS),
-    SelectRWSensor(276, "Prog3 Charge", options=PROG_CHARGE_OPTIONS),
-    SelectRWSensor(277, "Prog4 Charge", options=PROG_CHARGE_OPTIONS),
-    SelectRWSensor(278, "Prog5 Charge", options=PROG_CHARGE_OPTIONS),
-    SelectRWSensor(279, "Prog6 Charge", options=PROG_CHARGE_OPTIONS),
+    SelectRWSensor(274, "Prog1 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(275, "Prog2 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(276, "Prog3 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(277, "Prog4 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(278, "Prog5 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(279, "Prog6 charge", options=PROG_CHARGE_OPTIONS, bitmask=0x03),
+    SelectRWSensor(274, "Prog1 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
+    SelectRWSensor(275, "Prog2 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
+    SelectRWSensor(276, "Prog3 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
+    SelectRWSensor(277, "Prog4 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
+    SelectRWSensor(278, "Prog5 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
+    SelectRWSensor(279, "Prog6 mode", options=PROG_MODE_OPTIONS, bitmask=0x1C),
 )
 _SENSORS.extend(PROGRAM)
 PROG_VOLT = (
@@ -312,16 +329,22 @@ PROG_VOLT = (
 )
 _SENSORS.extend(PROG_VOLT)
 
+#############
+# Inverter settings
+#############
+_SENSORS.append(
+    NumberRWSensor(230, "Grid Charge Battery current", AMPS, min=0, max=185)
+)
 
 #############
 # Deprecated
 #############
-ALL_SENSORS: Dict[str, Sensor] = {s.id: s for s in _SENSORS}
+ALL_SENSORS: dict[str, Sensor] = {s.id: s for s in _SENSORS}
 
 
 def _deprecated() -> None:
     """Populate the deprecated sensors."""
-    dep_map: Dict[str, Sensor] = {
+    dep_map: dict[str, Sensor] = {
         "aux_power": Sensor(166, "AUX load", WATT, -1),
         "battery_temperature": TempSensor(182, "Temp Battery", CELSIUS, 0.1),
         "dc_transformer_temperature": TempSensor(
